#!/bin/bash
set -e

##
## Script used to gather information about the installation and environment 
## into a support package (tar.gz).
##
## Author: Michael Yara
## Maintainer: Michael Yara
## Last Modified: Nov. 14, 2015
##

# General support directories, files, and constants
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
SUPPORT_DIR=$SCRIPT_DIR/support
INSTALL_ID_FILE=$SCRIPT_DIR/install_id
NOW=`date +"%Y-%-m-%d-%H-%M-%S"`

# If the installation id file already exists, attempt
# to load the installation id out of it.
if [ -f $INSTALL_ID_FILE ]; then
	INSTALL_ID=$(cat $INSTALL_ID_FILE)
fi

# If an installation id does not yet exist generate one.
if [ -z "$INSTALL_ID" ]; then
	INSTALL_ID=$RANDOM
	echo $INSTALL_ID > $INSTALL_ID_FILE
fi

# Temporary package directory
PACKAGE_NAME=support_$INSTALL_ID\_$NOW
SUPPORT_PACKAGE_DIR=$SUPPORT_DIR/$PACKAGE_NAME

# Inner package directories
LOG_DIR=$SUPPORT_PACKAGE_DIR/logs
CONF_DIR=$SUPPORT_PACKAGE_DIR/conf
DB_DIR=$SUPPORT_PACKAGE_DIR/database
ENV_DIR=$SUPPORT_PACKAGE_DIR/environment
PBX_DIR=$SUPPORT_PACKAGE_DIR/pbx

# Asterisk manager files
MANAGER_FILE=/etc/asterisk/manager.conf
FPBX_MANAGER_FILE=/etc/asterisk/manager_additional.conf

# Load config-vars.sh so we know the JAVA_HOME path.
source $SCRIPT_DIR/config-vars.sh

# Instructs the running instance of the server to generate a thread dump.
function thread {
	local pid=`ps auxw | grep "com.xmlnamespace.panel.server.Server" | grep -v grep | awk ' { print $2 }'`
	if [ -z "$pid" ] ; then
		echo "Failed to generate thread dump: server not running"	
	else	
		kill -QUIT $pid

		# Sleep for a bit to allow the thread dump to generate
		# before we copy the logs.
		sleep 1
	fi	
}

# Copies log files to support package.
function logs {
	mkdir -p $LOG_DIR
	rsync --exclude='*.tar.gz' $SCRIPT_DIR/log/* $LOG_DIR
}

# Copies archived log files to support package.
function archived_logs {
	mkdir -p $LOG_DIR
	rsync --include='*.tar.gz' --include='*/' --exclude='*' $SCRIPT_DIR/log/* $LOG_DIR
}

# Copies the configuration files to the support package.
function config {
	mkdir -p $CONF_DIR
	rsync -r --exclude='**/servers/*/*.db' $SCRIPT_DIR/conf/* $CONF_DIR
}

# Copies the database files to the support package.
function database {
	mkdir -p $DB_DIR
	rsync -rm --include='**/servers/*/*.db' --include='*/' --exclude='*' $SCRIPT_DIR/conf/* $DB_DIR
}

# Copies environment information to the support package.
function environment {
	mkdir -p $ENV_DIR

	# Copy pid of running process
	local pid=`ps auxw | grep "com.xmlnamespace.panel.server.Server" | grep -v grep | awk ' { print $2 }'`
	echo $pid > $ENV_DIR/pid.txt

	# Copy CPU information
	cat /proc/cpuinfo > $ENV_DIR/cpu.txt

	# Copy Memory information
	local mem_file=$ENV_DIR/mem.txt
	free -m > $mem_file
	echo -e "\n" >> $mem_file
	cat /proc/meminfo >> $mem_file

	# Copy top snapshot
	top -n 1 -b > $ENV_DIR/top.txt

	# Copy netstat snapshot
	netstat -lanp > $ENV_DIR/netstat.txt

	# Copy jre information
	local jre_info=$(${JAVA_HOME}/bin/java -version 2>&1)
	echo $jre_info > $ENV_DIR/jre.txt

	# Copy os information
	uname -a > $ENV_DIR/os.txt

	# Copy jvm arguments
	cp $SCRIPT_DIR/jvm.args $ENV_DIR
}

# Copies PBX information to the support package.
function pbx {

	# If Asterisk is installed, copy its information
	local asterisk_bin=`command -v asterisk`
	if [ -z "$asterisk_bin" ] ; then
		echo "Excluding Asterisk information: Asterisk not installed"	
	else
		asterisk_info
	fi
}

# Copies Asterisk information to the support package.
function asterisk_info {
	mkdir -p $PBX_DIR

	# Copy the asterisk version information
	asterisk -rx "core show version" > $PBX_DIR/asterisk_version.txt

	# Copy sip peer information
	asterisk -rx "sip show peers" > $PBX_DIR/asterisk_sip_peers.txt 

	# Copy pjsip endpoint information
	asterisk -rx "pjsip show endpoints" > $PBX_DIR/asterisk_pjsip_endpoints.txt 

	# Copy channel information
	asterisk -rx "core show channels" > $PBX_DIR/asterisk_channels.txt

	# Copy queue information
	asterisk -rx "queue show" > $PBX_DIR/asterisk_queues.txt 

	# Copy confbridge information
	asterisk -rx "confbridge list" > $PBX_DIR/asterisk_confbridge.txt 

	# Copy meetme information
	asterisk -rx "meetme list" > $PBX_DIR/asterisk_meetme.txt

	# Copy manager.conf or manager_additional.conf file
	if [ -f $FPBX_MANAGER_FILE ]; then
		cp $FPBX_MANAGER_FILE $PBX_DIR
	elif [ -f $MANAGER_FILE ]; then
		cp $MANAGER_FILE $PBX_DIR
	else
		echo "Excluding Asterisk manager file: not found"
	fi
}

# Print usage.
function print_usage { 
	echo "Usage: $0 [-l] [-a] [-t] [-c] [-d] [-e] [-p] [-h]"
}

# Print help.
function print_help {
	echo "   -l : Include logs"
	echo "   -a : Include archived logs"
	echo "   -t : Generate thread dump"
	echo "   -c : Include configuration files"
	echo "   -d : Include database"
	echo "   -e : Include environment info"
	echo "   -p : Include PBX info"
	echo "   -h : Show help"
	echo -e "\n   If no arguments are given, all information will be collected"
}

# Determine given options.
# If no options are found, default to a predefined set of options.
if (( $# > 0 )); then
	while getopts ":latcdeph" flag; do
		case $flag in
			l) logs=true ;;
			a) archived_logs=true ;;
			t) thread=true ;;
			c) config=true ;;
			d) database=true ;;
			e) environment=true ;;
			p) pbx=true ;;
			h) 
			   print_usage
			   print_help
			   exit 0
			;;
			?)
			   echo "Invalid option: -$OPTARG"
			   print_usage
			   echo "$0 -h for help"
			   exit 1
			;;
		esac    
	done
else
	logs=true
	archived_logs=true
	thread=true
	config=true
	database=true
	environment=true
	pbx=true
fi

# Generate a thread dump first, if requested, so that 
# it will be included in the logs.
if [ "$thread" = true ] ; then
        thread
fi

# Copy each of the requested pieces of information to the support package directory.
if [ "$logs" = true ] ; then
	logs
fi

if [ "$archived_logs" = true ] ; then
	archived_logs
fi

if [ "$config" = true ] ; then
	config
fi

if [ "$database" = true ] ; then
	database
fi

if [ "$environment" = true ] ; then
	environment
fi

if [ "$pbx" = true ] ; then
	pbx
fi

# Create a compressed archive of the support package directory.
# Clean up the temporary directory after.
if [ -d $SUPPORT_PACKAGE_DIR ] ; then
	cd $SUPPORT_DIR
	tar -czf $PACKAGE_NAME.tar.gz $PACKAGE_NAME
	rm -rf $SUPPORT_PACKAGE_DIR
	echo "$SUPPORT_PACKAGE_DIR.tar.gz Created."
else 
	echo "No support package created."
	exit 100
fi

